#include <u.h>
#include <libc.h>
#include "db.h"

/*
*	a simple data base
*	ordered linear list database
*	1. improved programming interface
*	2. make double times faster (in theory)
*	3. data is ordered.
*
*	new usage is ...
*
*	DB *db;
*	db = db_gen();
*	db_put(db,"alice","18");
*	db_put(db,"bob","20");
*	print("%s\n",db_get(db,"alice"));
*	print("%s\n",db_get(db,"bob"));
*	print("%s\n",db_get(db,"carol"));
*	db_free(db);
*/

DB *db_gen(void)
{
	DB *q;
	q = (DB*)calloc(sizeof(DB),1);
	if(q == nil)
		sysfatal("no memory");
	return q;
}

void
db_put(DB *db, char *key, char *val)
{	/* the value may be nil */

	DB *p,*q;
	p = db;
	q = db->next;
	while(q && strcmp(q->key,key) < 0){
		p = q;
		q = q->next;
	}

	if(q && strcmp(q->key, key) == 0){
		q->val = val?strdup(val):nil;
		return;
	}
	/*	then strcmp(q->key, key) > 0
	*	we must insert new key	*/
	q  = (DB*)calloc(sizeof(DB),1);
	if(q == nil)
		sysfatal("no memory");
	q->key = strdup(key);
	q->val = val?strdup(val):nil;
	q->next = p->next;
	p->next = q;
}

char*
db_get(DB *db, char *key)
{
	DB *q;
	q = db->next;
	while(q && strcmp(q->key,key) < 0)
		q = q->next;
	if(q && strcmp(q->key,key) == 0)
		return q->val;
	return nil;
}

void
db_del(DB *db, char *key)
{
	DB *p, *q;
	p = db;
	q = db->next;
	while(q && strcmp(q->key,key) < 0){
		p = q;
		q = q->next;
	}
	if(q && strcmp(q->key, key) == 0){
		free(q->key);
		free(q->val);
		p->next = q->next;
		free(q);
	}
}

void
db_free(DB *db)
{
	DB *p, *q;
	q = db;
	while(q){
		free(q->key);
		free(q->val);
		p = q->next;
		free(q);
		q = p;
	}
}

void
db_print(DB *db)
{
	DB *q;
	q = db->next;
	while(q){
		print("%s %s\n", q->key, q->val);
		q = q->next;
	}
}

//#define TEST
#ifdef TEST
void
usage(void)
{
	fprint(2,"usage: 8.out\n");
	exits("usage");
}

void
main(int argc, char *argv[])
{
	DB *d;

	ARGBEGIN {
	default:
		usage();
	} ARGEND;

	d = db_gen();

	print("1\n");
	db_put(d,"bob","20");
	db_print(d);

	db_put(d,"alice","18");
	print("%p\n",d);
	db_print(d);


	print("%s\n",db_get(d,"alice"));
	print("%s\n",db_get(d,"bob"));
	print("%s\n",db_get(d,"carol"));

	print("2\n");
	db_put(d,"alice","23");
	db_print(d);

	print("1\n");
	db_put(d,"carol","25");
	db_print(d);


	print("6\n");
	db_del(d, "carol");
	db_print(d);

	print("5\n");
	db_del(d, "alice");
	db_print(d);

	print("3\n");
	db_del(d, "bob");
	db_print(d);

	print("4\n");
	db_del(d, "bob");
	db_print(d);

	db_free(d);
}
#endif
